@@ -37,6 +37,8 @@
 #include "memory.h"
 #include "dynload.h"
 
+#include "bithelp.h"
+
 
 typedef struct {
     u32 A,B,C,D;	  /* chaining variables */
@@ -104,15 +106,11 @@ transform( MD5_CONTEXT *ctx, byte *data )
   do								    \
     {								    \
       a += FF (b, c, d) + (*cwp++) + T; 	    \
-      CYCLIC (a, s);						    \
+      a = rol(a, s);						    \
       a += b;							    \
     }								    \
   while (0)
 
-  /* It is unfortunate that C does not provide an operator for
-     cyclic rotation.  Hope the C compiler is smart enough.  */
-#define CYCLIC(w, s) (w = (w << s) | (w >> (32 - s)))
-
     /* Before we start, one word about the strange constants.
        They are defined in RFC 1321 as
 
@@ -142,7 +140,7 @@ transform( MD5_CONTEXT *ctx, byte *data )
     do								      \
       { 							      \
 	a += f (b, c, d) + correct_words[k] + T;		      \
-	CYCLIC (a, s);						      \
+	a = rol(a, s);						      \
 	a += b; 						      \
       } 							      \
     while (0)
@@ -346,10 +344,10 @@ md5_get_info( int algo, size_t *contextsize,
     *r_asnoid = asn;
     *r_asnlen = DIM(asn);
     *r_mdlen = 16;
-    *r_init  = (void (*)(void *))md5_init;
-    *r_write = (void (*)(void *, byte*, size_t))md5_write;
-    *r_final = (void (*)(void *))md5_final;
-    *r_read  = (byte *(*)(void *))md5_read;
+    *(void  (**)(MD5_CONTEXT *))r_init		       = md5_init;
+    *(void  (**)(MD5_CONTEXT *, byte*, size_t))r_write = md5_write;
+    *(void  (**)(MD5_CONTEXT *))r_final 	       = md5_final;
+    *(byte *(**)(MD5_CONTEXT *))r_read		       = md5_read;
 
     return "MD5";
 }
@@ -358,7 +356,7 @@ md5_get_info( int algo, size_t *contextsize,
 #ifndef IS_MODULE
 static
 #endif
-const char * const gnupgext_version = "MD5 ($Revision: 1.14 $)";
+const char * const gnupgext_version = "MD5 ($Revision: 1.15.2.1 $)";
 
 static struct {
     int class;
