@@ -1,5 +1,5 @@
 /* helptext.c  - English help texts
- *	Copyright (C) 1998 Free Software Foundation, Inc.
+ *	Copyright (C) 1998, 1999, 2000 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -210,7 +210,7 @@ static struct helptexts { const char *key; const char *help; } helptexts[] = {
 )},
 
 { "detached_signature.filename", N_(
- "Give the name fo the file to which the signature applies"
+ "Give the name of the file to which the signature applies"
 )},
 
 /* openfile.c (overwrite_filep) */
@@ -224,6 +224,29 @@ static struct helptexts { const char *key; const char *help; } helptexts[] = {
  "file (which is shown in brackets) will be used."
 )},
 
+/* revoke.c (ask_revocation_reason) */
+{ "ask_revocation_reason.code", N_(
+ "You should specify a reason for the certification.  Depending on the\n"
+ "context you have the ability to choose from this list:\n"
+ "  \"Key has been compromised\"\n"
+ "      Use this if you have a reason to believe that unauthorized persons\n"
+ "      got access to your secret key.\n"
+ "  \"Key is superseded\"\n"
+ "      Use this if you have replaced this key with a newer one.\n"
+ "  \"Key is no longer used\"\n"
+ "      Use this if you have retired this key.\n"
+ "  \"User ID is no longer valid\"\n"
+ "      Use this to state that the user ID should not longer be used;\n"
+ "      this is normally used to mark an email address invalid.\n"
+)},
+
+/* revoke.c (ask_revocation_reason) */
+{ "ask_revocation_reason.text", N_(
+ "If you like, you can enter a text describing why you issue this\n"
+ "revocation certificate.  Please keep this text concise.\n"
+ "An empty line ends the text.\n"
+)},
+
 /* end of list */
 { NULL, NULL } };
 
