@@ -24,6 +24,7 @@
 #include <string.h>
 #include <errno.h>
 #include <unistd.h>
+#include <signal.h>
 #ifdef USE_SHM_COPROCESSING
   #ifdef USE_CAPABILITIES
     #include <sys/capability.h>
@@ -46,6 +47,10 @@
 #include "i18n.h"
 #include "cipher.h" /* for progress functions */
 
+#define CONTROL_D ('D' - 'A' + 1)
+
+
+
 static int fd = -1;
 #ifdef USE_SHM_COPROCESSING
   static int shm_id = -1;
@@ -151,6 +156,7 @@ write_status_text ( int no, const char *text)
       case STATUS_DELETE_PROBLEM : s = "DELETE_PROBLEM\n"; break;
       case STATUS_PROGRESS	 : s = "PROGRESS\n"; break;
       case STATUS_SIG_CREATED	 : s = "SIG_CREATED\n"; break;
+      case STATUS_SESSION_KEY	 : s = "SESSION_KEY\n"; break;
       default: s = "?\n"; break;
     }
 
@@ -339,6 +345,28 @@ do_shm_get( const char *keyword, int hidden, int bool )
 
 #endif /* USE_SHM_COPROCESSING */
 
+static int
+myread(int fd, void *buf, size_t count)
+{
+    int rc;
+    do {
+        rc = read( fd, buf, count );
+    } while ( rc == -1 && errno == EINTR );
+    if ( !rc && count ) {
+        static int eof_emmited=0;
+        if ( eof_emmited < 3 ) {
+            *(char*)buf = CONTROL_D;
+            rc = 1;
+            eof_emmited++;
+        }
+        else { /* Ctrl-D not caught - do something reasonable */
+            raise (SIGHUP); /* no more input data */
+        }
+    }    
+    return rc;
+}
+
+
 
 /****************
  * Request a string from the client over the command-fd
@@ -364,8 +392,14 @@ do_get_from_fd( const char *keyword, int hidden, int bool )
 		i=0;
 	}
 	/* Hmmm: why not use our read_line function here */
-	if( read( fd, string+i, 1) != 1 || string[i] == '\n' )
-	    break;
+	if( myread( opt.command_fd, string+i, 1) != 1 || string[i] == '\n'  )
+            break;
+        else if ( string[i] == CONTROL_D ) {
+            /* found ETX - cancel the line and return a sole ETX */
+            string[0] = CONTROL_D;
+            i=1;
+            break;
+        }
     }
     string[i] = 0;
 
