@@ -1,5 +1,5 @@
 /* trustdb.h - Trust database
- *	Copyright (C) 1998 Free Software Foundation, Inc.
+ *	Copyright (C) 1998, 1999, 2000 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -34,6 +34,7 @@
 /* trust values not covered by the mask */
 #define TRUST_FLAG_REVOKED 32 /* r: revoked */
 #define TRUST_FLAG_SUB_REVOKED 64
+#define TRUST_FLAG_DISABLED 128 /* d: key/uid disabled */
 
 
 #define PREFTYPE_SYM	  1
@@ -48,6 +49,7 @@ void check_trustdb( const char *username );
 void update_trustdb( void );
 int setup_trustdb( int level, const char *dbname );
 void init_trustdb( void );
+void sync_trustdb( void );
 int check_trust( PKT_public_key *pk, unsigned *r_trustlevel,
 		 const byte* nh, int (*add_fnc)(ulong), unsigned *retflgs );
 int query_trust_info( PKT_public_key *pk, const byte *nh );
@@ -63,8 +65,9 @@ int keyid_from_lid( ulong lid, u32 *keyid );
 ulong lid_from_keyblock( KBNODE keyblock );
 int query_trust_record( PKT_public_key *pk );
 int clear_trust_checked_flag( PKT_public_key *pk );
-int insert_trust_record( PKT_public_key *pk );
 int update_trust_record( KBNODE keyblock, int fast, int *modified );
+int insert_trust_record( KBNODE keyblock );
+int insert_trust_record_by_pk( PKT_public_key *pk );
 int update_ownertrust( ulong lid, unsigned new_trust );
 int trust_letter( unsigned value );
 
