@@ -23,15 +23,20 @@
 #include <stdlib.h>
 #include <string.h>
 #include <errno.h>
+#include <ctype.h>
 #include <assert.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <unistd.h>
+#ifdef HAVE_DOSISH_SYSTEM
+  #include <fcntl.h> /* for setmode() */
+#endif
 
 #include "memory.h"
 #include "util.h"
 #include "iobuf.h"
 
+
 typedef struct {
     FILE *fp;	   /* open file handle */
     int  print_only_name; /* flags indicating that fname is not a real file*/
@@ -56,6 +61,7 @@ typedef struct {
     int eof;
 } block_filter_ctx_t;
 
+static int special_names_enabled;
 
 static int underflow(IOBUF a);
 
@@ -467,7 +473,7 @@ int
 iobuf_close( IOBUF a )
 {
     IOBUF a2;
-    size_t dummy_len;
+    size_t dummy_len=0;
     int rc=0;
 
     if( a && a->directfp ) {
@@ -564,6 +570,32 @@ iobuf_temp_with_content( const char *buffer, size_t length )
     return a;
 }
 
+void
+iobuf_enable_special_filenames ( int yes )
+{
+    special_names_enabled = yes;
+}
+
+/*
+ * see whether the filename has the for "-&nnnn", where n is a
+ * non-zero number.
+ * Returns this number or -1 if it is not the case.
+ */
+static int
+check_special_filename ( const char *fname )
+{
+    if ( special_names_enabled
+         && fname && *fname == '-' && fname[1] == '&' ) {
+        int i;
+
+        fname += 2;
+        for (i=0; isdigit (fname[i]); i++ )
+            ;
+        if ( !fname[i] ) 
+            return atoi (fname);
+    }
+    return -1;
+}
 
 /****************
  * Create a head iobuf for reading from a file
@@ -577,12 +609,18 @@ iobuf_open( const char *fname )
     file_filter_ctx_t *fcx;
     size_t len;
     int print_only = 0;
+    int fd;
 
     if( !fname || (*fname=='-' && !fname[1])  ) {
-	fp = stdin; /* fixme: set binary mode for msdoze */
+	fp = stdin;
+      #ifdef HAVE_DOSISH_SYSTEM
+	setmode ( fileno(fp) , O_BINARY );
+      #endif
 	fname = "[stdin]";
 	print_only = 1;
     }
+    else if ( (fd = check_special_filename ( fname )) != -1 )
+        return iobuf_fdopen ( fd, "rb" );
     else if( !(fp = fopen(fname, "rb")) )
 	return NULL;
     a = iobuf_alloc(1, 8192 );
@@ -643,12 +681,18 @@ iobuf_create( const char *fname )
     file_filter_ctx_t *fcx;
     size_t len;
     int print_only = 0;
+    int fd;
 
     if( !fname || (*fname=='-' && !fname[1]) ) {
 	fp = stdout;
+      #ifdef HAVE_DOSISH_SYSTEM
+	setmode ( fileno(fp) , O_BINARY );
+      #endif
 	fname = "[stdout]";
 	print_only = 1;
     }
+    else if ( (fd = check_special_filename ( fname )) != -1 )
+        return iobuf_fdopen ( fd, "wb" );
     else if( !(fp = fopen(fname, "wb")) )
 	return NULL;
     a = iobuf_alloc(2, 8192 );
@@ -671,6 +715,7 @@ iobuf_create( const char *fname )
 /****************
  * append to an iobuf; if the file does not exist, create it.
  * cannot be used for stdout.
+ * Note: This is not used.
  */
 IOBUF
 iobuf_append( const char *fname )
@@ -741,7 +786,10 @@ iobuf_fopen( const char *fname, const char *mode )
     int print_only = 0;
 
     if( !fname || (*fname=='-' && !fname[1])  ) {
-	fp = stdin; /* fixme: set binary mode for msdoze */
+	fp = stdin;
+      #ifdef HAVE_DOSISH_SYSTEM
+	setmode ( fileno(fp) , O_BINARY );
+      #endif
 	fname = "[stdin]";
 	print_only = 1;
     }
@@ -985,7 +1033,7 @@ underflow(IOBUF a)
 
 	}
 	if( a->use == 1 && rc == -1 ) { /* EOF: we can remove the filter */
-	    size_t dummy_len;
+	    size_t dummy_len=0;
 
 	    /* and tell the filter to free itself */
 	    if( (rc = a->filter(a->filter_ov, IOBUFCTRL_FREE, a->chain,
@@ -1280,7 +1328,7 @@ iobuf_flush_temp( IOBUF temp )
  * Setting the limit to 0 disables this feature.
  */
 void
-iobuf_set_limit( IOBUF a, unsigned long nlimit )
+iobuf_set_limit( IOBUF a, off_t nlimit )
 {
     if( nlimit )
 	a->nofast |= 1;
@@ -1296,7 +1344,7 @@ iobuf_set_limit( IOBUF a, unsigned long nlimit )
 /****************
  * Return the length of an open file
  */
-u32
+off_t
 iobuf_get_filelength( IOBUF a )
 {
     struct stat st;
@@ -1304,8 +1352,8 @@ iobuf_get_filelength( IOBUF a )
     if( a->directfp )  {
 	FILE *fp = a->directfp;
 
-	if( !fstat(fileno(fp), &st) )
-	    return st.st_size;
+       if( !fstat(fileno(fp), &st) )
+           return st.st_size;
 	log_error("fstat() failed: %s\n", strerror(errno) );
 	return 0;
     }
@@ -1316,7 +1364,7 @@ iobuf_get_filelength( IOBUF a )
 	    file_filter_ctx_t *b = a->filter_ov;
 	    FILE *fp = b->fp;
 
-	    if( !fstat(fileno(fp), &st) )
+           if( !fstat(fileno(fp), &st) )
 		return st.st_size;
 	    log_error("fstat() failed: %s\n", strerror(errno) );
 	    break;
@@ -1328,27 +1376,55 @@ iobuf_get_filelength( IOBUF a )
 /****************
  * Tell the file position, where the next read will take place
  */
-ulong
+off_t
 iobuf_tell( IOBUF a )
 {
     return a->ntotal + a->nbytes;
 }
 
 
+#if !defined(HAVE_FSEEKO) && !defined(fseeko)
+
+#ifdef HAVE_LIMITS_H
+# include <limits.h>
+#endif
+#ifndef LONG_MAX
+# define LONG_MAX ((long) ((unsigned long) -1 >> 1))
+#endif
+#ifndef LONG_MIN
+# define LONG_MIN (-1 - LONG_MAX)
+#endif
+
+/****************
+ * A substitute for fseeko, for hosts that don't have it.
+ */
+static int
+fseeko( FILE *stream, off_t newpos, int whence )
+{
+    while( newpos != (long) newpos ) {
+       long pos = newpos < 0 ? LONG_MIN : LONG_MAX;
+       if( fseek( stream, pos, whence ) != 0 )
+           return -1;
+       newpos -= pos;
+       whence = SEEK_CUR;
+    }
+    return fseek( stream, (long)newpos, whence );
+}
+#endif
 
 /****************
  * This is a very limited implementation. It simply discards all internal
  * buffering and removes all filters but the first one.
  */
 int
-iobuf_seek( IOBUF a, ulong newpos )
+iobuf_seek( IOBUF a, off_t newpos )
 {
     file_filter_ctx_t *b = NULL;
 
     if( a->directfp ) {
 	FILE *fp = a->directfp;
-	if( fseek( fp, newpos, SEEK_SET ) ) {
-	    log_error("can't seek to %lu: %s\n", newpos, strerror(errno) );
+       if( fseeko( fp, newpos, SEEK_SET ) ) {
+           log_error("can't seek: %s\n", strerror(errno) );
 	    return -1;
 	}
 	clearerr(fp);
@@ -1362,8 +1438,8 @@ iobuf_seek( IOBUF a, ulong newpos )
 	}
 	if( !a )
 	    return -1;
-	if( fseek( b->fp, newpos, SEEK_SET ) ) {
-	    log_error("can't seek to %lu: %s\n", newpos, strerror(errno) );
+       if( fseeko( b->fp, newpos, SEEK_SET ) ) {
+           log_error("can't seek: %s\n", strerror(errno) );
 	    return -1;
 	}
     }
