@@ -100,6 +100,9 @@ lock_pool( void *p, size_t n )
 	  #ifdef ENOSYS  /* Some SCOs return this (function not implemented) */
 	    && errno != ENOSYS
 	  #endif
+          #ifdef ENOMEM  /* Linux can return this */
+            && errno != ENOMEM
+          #endif
 	  )
 	    log_error("can't lock memory: %s\n", strerror(err));
 	show_warning = 1;
@@ -142,6 +145,9 @@ lock_pool( void *p, size_t n )
 	  #ifdef ENOSYS  /* Some SCOs return this (function not implemented) */
 	    && errno != ENOSYS
 	  #endif
+          #ifdef ENOMEM  /* Linux can return this */
+            && errno != ENOMEM
+          #endif
 	  )
 	    log_error("can't lock memory: %s\n", strerror(err));
 	show_warning = 1;
@@ -347,9 +353,11 @@ secmem_realloc( void *p, size_t newsize )
     if( newsize < size )
 	return p; /* it is easier not to shrink the memory */
     a = secmem_malloc( newsize );
-    memcpy(a, p, size);
-    memset((char*)a+size, 0, newsize-size);
-    secmem_free(p);
+    if ( a ) {
+        memcpy(a, p, size);
+        memset((char*)a+size, 0, newsize-size);
+        secmem_free(p);
+    }
     return a;
 }
 
