@@ -424,7 +424,8 @@ utf8_to_native( const char *string, size_t length )
 			  case	 0 : n++; if( p ) *p++ = '0'; break;
 			  default: n += 3;
 				   sprintf( p, "x%02x", *s );
-				   p += 3;
+                                   if ( p )
+                                       p += 3;
 				   break;
 			}
 		    }
@@ -559,6 +560,18 @@ strlwr(char *s)
 }
 #endif
 
+#ifndef HAVE_STRCASECMP
+int
+strcasecmp( const char *a, const char *b )
+{
+    for( ; *a && *b; a++, b++ ) {
+	if( *a != *b && toupper(*a) != toupper(*b) )
+	    break;
+    }
+    return *(const byte*)a - *(const byte*)b;
+}
+#endif
+
 /****************
  * mingw32/cpd has a memicmp()
  */
